@@ -3,16 +3,16 @@ use strict;
 use warnings;
 
 package Mxpress::PDF {
-	our $VERSION = '0.08';
+	our $VERSION = '0.09';
 	use Zydeco (
-		version	=> '0.08',
+		version	=> '0.09',
 		authority => 'cpan:LNATION',
 	);
 	use Colouring::In;
 	use constant mm => 25.4 / 72;
 	use constant pt => 1;
 	class File (HashRef $args) {
-		my @plugins = (qw/font line box circle pie ellipse text title subtitle subsubtitle toc image/, ($args->{plugins} ? @{$args->{plugins}} : ()));
+		my @plugins = (qw/font line box circle pie ellipse text title subtitle subsubtitle toc image form field annotation/, ($args->{plugins} ? @{$args->{plugins}} : ()));
 		for my $p (@plugins) {
 			my $meth = sprintf('_store_%s', $p);
 			has {$meth} (type => Object);
@@ -477,6 +477,7 @@ package Mxpress::PDF {
 			}
 		}
 		class +Text {
+			has text (type => Object);
 			has font (type => Object);
 			has paragraph_space (type => Num);
 			has paragraphs_to_columns (type => Bool);
@@ -526,7 +527,7 @@ package Mxpress::PDF {
 					$self->file->page->columns(scalar grep { ($_ =~ m/\w/) } @paragraphs);
 					$page_column = 1;
 				}
-				my $text = $self->file->page->current->text;
+				my $text = $self->text($self->file->page->current->text);
 				$text->font( $self->font->load, $self->font->size/pt );
 				$text->fillcolor( $self->font->colour );
 				my ($total_width, $space_width, %width) = $self->_calculate_widths($string, $text);
@@ -589,7 +590,7 @@ package Mxpress::PDF {
 						if (@paragraph) {
 							$ypos -= $l if @paragraph;
 						} elsif ($self->pad) {
-							my $pad_end = $self->pad_end;
+							my $pad_end = $self->pad_end || '';
 							my $pad = sprintf ("%s%s",
 								$self->pad x int(((
 									(((($lw + $wordspace) - $line_width) - $text->advancewidth($self->pad . $pad_end)))
@@ -869,6 +870,74 @@ package Mxpress::PDF {
 				return ($x, $y, $width, $height);
 			}
 		}
+		class +Annotation {
+			has type (type => Str);
+			has w (type => Num);
+			has h (type => Num);
+			has open (type => Bool);
+			has rect (type => ArrayRef);
+			has border (type => ArrayRef);
+			factory annotation (Object $file, Map %args) {
+				return $class->new(
+					file => $file,
+					padding => 0,
+					type => 'text',
+					open => 0,
+					w => 0,
+					h => 0,
+					%args
+				);
+			}
+			method add (Str $text, %args) {
+				my $annotation = $self->file->page->current->annotation;
+				return $self->_add_annotation($self->type, $text, %args);
+			}
+			method _add_annotation (Str $type, Str $cont, Map %args) {
+				$self->set_attrs(%args);
+				my $annotation = $self->file->page->current->annotation;
+				my @xy = $self->parse_position(($self->position || []), 1);
+				$annotation->$type($cont,
+					-rect => $self->rect || [@xy, $self->w, $self->h],
+					-open => $self->open,
+					-border => $self->border || [@xy, $self->w]
+				); # :/
+				return $self->file;
+			}
+		}
+		class +Form {
+			has xo (type => Object);
+			factory form (Object $file, Map %args) {
+				return $class->new(
+					file => $file,
+					forms => [],
+					padding => $args{padding} || 0,
+				);
+			}
+			method add {
+				my $form = $self->xo(
+					$self->file->pdf->xo_form()
+				);
+				$self->position([$self->parse_position([], 1)]);
+				return $self->file;
+			}
+			method end (Map %args) {
+				my $position = $self->position;
+
+				return $self->file;
+			}
+		}
+		class +Field {
+			extends Plugin::Text;
+			factory field (Object $file, Map %args) {
+				$args{pad} ||= '_';
+				$class->generic_new($file, %args);
+				# todo Attempt2 when brain != hangover |--[fillable forms]--|
+			}
+			around add (Str $text, Map %args) {
+				my $file = $self->$next($text, %args);
+				return $file;
+			}
+		}
 	}
 	class Factory {
 		use PDF::API2;
@@ -897,7 +966,7 @@ Mxpress::PDF - PDF
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =cut
 
@@ -947,8 +1016,9 @@ This is experimental and may yet still change.
 			margin_bottom => 3
 		},
 		text => {
-			font => { align => 'justify', colour => '#fff' },
-			margin_bottom => 3
+			font => { colour => '#fff' },
+			margin_bottom => 3,
+			align => 'justify'
 		},
 	)->add_page;
 
@@ -979,6 +1049,8 @@ This is experimental and may yet still change.
 		)->text->add( $gen_text->(1000) );
 	}
 
+	$pdf->save();
+
 =head1 Description
 
 This module currently allows you to easily create a PDF. Why? For fun.
@@ -2237,6 +2309,65 @@ or
 
 	$img->add($image_file_path, %image_attrs)
 
+=head1 Annotation
+
+Mxpress::PDF::Plugin::Annotation extends Mxpress::PDF::Plugin and is for adding annotations to a
+Mxpress::PDF::Page.
+
+You can pass default attributes when instantiating the file object.
+
+	Mxpress::PDF->add_file($filename,
+		annotation => { %annotation_attrs },
+	);
+
+or when calling the objects add method.
+
+	$file->annotation->add(
+		%annotation_attrs
+	);
+
+	my $annotation = $annotation->annotation;
+
+=head2 Attributes
+
+The following attributes can be configured for a Mxpress::PDF::Plugin::Annotation object, they are all optional.
+
+	$annotation->$attrs();
+
+=head3 type (type => Num)
+
+The type of annotation text|file|.
+
+	$annotation->type;
+
+=head3 w (type => Num)
+
+The width of the annotation.
+
+	$annotation->w;
+
+=head3 h (type => Num)
+
+The hieght of the annotation.
+
+	$annotation->h;
+
+=head2 open (type => Bool)
+
+Toggle whether annotation is open.
+
+	$annotation->open;
+
+=head2 Methods
+
+The following methods can be called from a Mxpress::PDF::Plugin::Annotation Object.
+
+=head3 add
+
+Add an annotation to the current Mxpress::PDF::Page.
+
+	$annotation->add('add some text', %annotation_attrs)
+
 =head1 AUTHOR
 
 LNATION, C<< <thisusedtobeanemail at gmail.com> >>
